#if (__GNUC__ * 100 + __GNUC_MINOR__) < 304
#error "This library requires AVR-GCC 3.4 or later, update to newer AVR-GCC compiler !"
#endif

#ifndef F_CPU
	#define F_CPU 8000000UL
#endif
#ifndef TRUE
	#define TRUE 1
	#define FALSE 0
#endif


#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include <inttypes.h>
//#include <compat/twi.h>
#include <twimaster.c>

#define UART_UBRR_CALC(BAUD_,FREQ_) ((FREQ_)/((BAUD_)*16L)-1)
#define UART_BAUD_RATE 9600
#define UART_BUFFER      10

void pause( uint16_t ms ) {
	/* pauses the program for the given time;
	* ms: time in milliseconds (0 - 65535)
	*/

	uint16_t i;

	for( i = 0; i < ms / 63; i++ ) {
		_delay_ms( 63 );
	}

	_delay_ms( ms % 63 );
}



/* UART */

inline void uart_init( uint8_t tx, uint8_t rx ) {
	/* initializes UART communication;
	* tx: sending should be activated
	* rx: receiving should be activated
	*/

	uint16_t baudrate;

	// Baud-Rate setzen
	baudrate = UART_BAUD_RATE / 2;
	UBRRH    = (uint8_t) ( UART_UBRR_CALC( baudrate, F_CPU ) >> 8 );
	UBRRL    = (uint8_t) UART_UBRR_CALC( baudrate, F_CPU );

	// Ggf. TX und RX anschalten
	UCSRA |= ( 1 << U2X );
	UCSRB |= ( 1 << RXCIE );
	if( tx )
		UCSRB |= ( 1 << TXEN );

	if( rx )
		UCSRB |= ( 1 << RXEN );

	sei();
	pause( 1 );
}

void uart_putc( char c ) {
	/* send one char per UART
	*/

	while( !( UCSRA & ( 1 << UDRE ) ) );
	UDR = c;
}

void uart_puts( char *data ) {
	/* send string per UART
	*/

	while( *data ) {
		uart_putc( *data );
		data++;
	}
}

void uart_puti( uint16_t data ) {
	/* send integer per UART
	*/

	char buffer[ 7 ];
	uart_puts( utoa( data, buffer, 10 ) );
}

void uart_putf( float data ) {
	/* send float per UART
	   BUG: unnötige Leerzeichen am Anfang?
	*/

	char buffer[ 7 ];
	uart_puts( dtostrf( data, 6, 2, buffer ) );
}

#ifdef UART_BUFFER
	volatile uint8_t uart_buffer[ UART_BUFFER ];
	//volatile int8_t uart_count = 0;	<- evtl. volatile???
	volatile int8_t uart_count = 0;

    uint8_t uart_getc( void ) {
		/* output next char in cache
		*/

		if( uart_count == 0 )
			return( 0 );

		return( uart_buffer[ --uart_count ] );
	}

	ISR( SIG_UART_RECV ) {
		/* INTERRUPT: receive char from UART and save it the buffer
		*/
		if( uart_count < UART_BUFFER )
			uart_buffer[ uart_count++ ] = UDR;
	}
#endif

void init_counter1( void ) {
	DDRD &= ~( 1 << DDD5 );		// Clear the PD5(T1) pin
								// PD5(T1) is now an input
	// Turn on the counter, Clock on Rise
	TCCR1B |= ( 1 << CS12 ) | ( 1 << CS11 ) | ( 1 << CS10 );
}

float get_freq( void ) {
	float r_freq = 0.0;
	TCNT1 = 0;
	pause( 100 );				// 100 ms messen
	r_freq = TCNT1;
	r_freq = r_freq * 10;
	return( r_freq );			// Ergebnis * 10 nehmen
}


int main( void ) {
	float sens = 0.0;			// sensitivity One-Byte-Value
	float senslsb = 0.0;
	float offset = 0.0;			// offset Two-Byte-Value
	float offlsb = 0.0;			// LSB von offset
	float rh = 0.0;				// Relative Luftfeuchtigkeit

	init_counter1();			// Counter initialisieren und starten

	uart_init( 1, 1 );			// COM Port initialisieren
	sei();
	i2c_init();					// I²C initialisieren
	i2c_start( 0xA2 );			// I²C Übertragung starten mit S
	i2c_write( 0x0A );			// Adresse die als erstes Ausgelesen werden soll
	i2c_start( 0xA3 );			// Daten anfordern
	sens = i2c_readAck();		// 0x0A MSB von sens
	senslsb = i2c_readAck();	// 0x0B LSB von sens
	offset = i2c_readAck();		// 0x0C MSB von offset
	offlsb = i2c_readNak();		// 0x0D LSB von offset, es werden keine weiter Daten mehr angefordert
	i2c_stop();					// I²C Übertragung beenden

	offset = offset * 256;		// Stellenwert von offset-MSB berücksichten
	offset = offset + offlsb;	// MSB und LSB von Offset zusammenfügen
	sens = sens * 256;			// Stellenwert von sens-MSB berücksichten
	sens = sens + senslsb;		// MSB und LSB von sens zusammenfügen

	uart_puts( "I2C -> Sens: " );
	// Ausgelesene Werte von Sens und Offset als
	uart_putf( sens );	// Dezimal-Werte Ausgeben
	uart_puts( "; Offset: " );
	uart_putf( offset );
	uart_puts( ";\n\r" );
	uart_puts( "System bereit.\n\r" );

	char buf;

	while( 1 ) {

		rh = ( offset - get_freq() );
		rh = rh * sens;
		rh = rh / 4096;
		uart_puts( "\n\rLuftfeuchtigkeit: " );
		uart_putf( rh );

		pause( 1000 );	// ein kleines paeuschen
	}

	return( 0 );
}
